// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2008  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Apologies for the lack of comments and the horrible code structure. I will fix
// this soon. If you have any questions feel free to email me at:
// ali@arkitus.com
// ------------------------------------------------------------------------------

class DraggableLine
{
  int     state; // 0 - default, 1 - highlighted, 2 - selected
  Vector  p1;
  Vector  p2;
  Vector  axis;
  Vector  min;
  Vector  max;
  
  DraggableLine
  ( Vector p1,
    Vector p2,
    Vector axis,
    Vector min,
    Vector max )
  {
    this.p1 = p1;
    this.p2 = p2;
    this.axis = axis;
    this.min = min;
    this.max = max;
  }
  
  boolean update
  ( Cursor   cursor,
    boolean  token )
  {
    float cx = cursor.position.x;
    float cy = cursor.position.y;
    
    float maxx = max( p1.x, p2.x );
    float minx = min( p1.x, p2.x );
    float maxy = max( p1.y, p2.y );
    float miny = min( p1.y, p2.y );
    float z = DRAGGABLE_SNAP;

    if ( token == true )
    {
      state = 0;
      return true;
    }
    
    if ( state == 1 && cursor.pressed && !cursor.selected)
    {
      state = 2;
      cursor.selected = true;
      return true;
    }
    
    if ( state == 2 )
    {
      if ( !cursor.pressed )
      {
        state = 1;
        cursor.selected = false;
        freshDraw();
      }
    }
    else
    {
      Vector diff = p2.sub(p1);
      float d = abs( diff.x * ( p2.y - cy ) - ( p2.x - cx ) * diff.y ) / diff.length();
      
      if ( cx > minx - z && cx < maxx + z && cy > miny - z && cy < maxy + z  && d < z && !cursor.selected )
      {
        state = 1;
        freshDraw();
      }
      else
      {
        if ( state != 0 )
        {
          state = 0;
          freshDraw();
        }
      }
    }
    
    if ( state == 2 )
    {
      Vector diff;
      Vector p3;
      Vector p4;
      Vector p5 = new Vector();
      float  len;
      
      diff = new Vector( cx - p1.x, cy - p1.y, 0 );
      len = diff.dot( axis ) / axis.length();
      
      p3 = p1.add( axis.mul( len ) );
      p4 = p2.add( axis.mul( len ) );
      
      p5.x = ( p3.x < min.x ) ? min.x : p3.x;
      p5.x = ( p3.x > max.x ) ? max.x : p5.x;
      p5.y = ( p3.y < min.y ) ? min.y : p3.y;
      p5.y = ( p3.y > max.y ) ? max.y : p5.y;
     
      p2.x += p5.x - p1.x;
      p2.y += p5.y - p1.y;
      p1.x = p5.x;
      p1.y = p5.y;
    }
    
    if ( state == 1 )
    {
      return true;
    }
    else
    {
      return false;
    }
  }
  
  void draw
  ()
  {
    switch ( state )
    {
      case 0:
        stroke( 200, 200, 200 ); break;
      case 1:
        stroke( 100, 100, 100 ); break;
      case 2:
        stroke( 200, 0, 0 ); break;
    }
    
    strokeWeight( 5 );
    line( p1.x, p1.y, p2.x, p2.y );
  }
  
  Vector intersect( DraggableLine line )
  {
    Vector r = new Vector();
    float x1, x2, x3, x4;
    float y1, y2, y3, y4;
    float ua;
    float ub;
    
    x1 = p1.x;
    y1 = p1.y;
    
    x2 = p2.x;
    y2 = p2.y;
    
    x3 = line.p1.x;
    y3 = line.p1.y;
    
    x4 = line.p2.x;
    y4 = line.p2.y;
    
    ua = ((x4 - x3)*(y1 - y3) - (y4-y3)*(x1-x3))/((y4 - y3)*(x2 - x1) - (x4 - x3)*(y2 - y1));
    // ub = ((x2 - x1)*(y1 - y3) - (y2-y1)*(x1-x3))/((y4 - y3)*(x2 - x1) - (x4 - x3)*(y2 - y1));
    
    r.x = x1 + ua * ( x2 - x1 );
    r.y = y1 + ua * ( y2 - y1 );
    
    return r;
  }
}
